/* -*- C -*- 
c     Program cavity
c     Original Coded By K.Yanagi (Fortran)
c     Arranged By H. Kogo   1991. 6.29 (Fortran)
c              By T. Nemoto 1993.10    Version 4.0 (Fortran)
c                           1994.10    Version 4.1 (Fortran)
c                           1994.11    Version 4.5 (Fortran)
c                           1997.12    Version 5.0 (C)
c                           2002.12    Version 5.1 (C)
*/

#include "cavity.h"
#include <stdlib.h>
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

static int CavMainLoop(void);
static int PlotVDW(int nSec,int iZNum);
static int ICave(void);
static void setpl(void);
static void CheckRange(void);
static double PlVol(void);
static int AutoExpandRange(int nSec,int l,double xp,double yp);

static double zsec;

void error(char *mess) {
  fprintf(stderr,"#Error:%s\n",mess);
  CavPlotComment(mess);
  iErr=1;
  CavTrailer();
  exit(1);
}

int main(int argc,char **argv) {
  int iv;

  fprintf(stderr,"Cavity version %s\n",VERSION);
  /*** read file , etc... ***/
  CavInit();
  /*** iview loop ***/
  iv=1;
  if (iView==0) {
    /*      ... Mesh Mode */
    iv=3;
    iView=3;
  }
  CavPlotMeshDirection(iView);

  do {
    CavPlotMeshBegin(iView);
    /*     ... set axis, etc.  */
    cavin2();
    CavPlotMatrix();
    /*** cavity Main ***/
    /*     ... plot one view*/
    CavMainLoop();
    CavPlotMeshEnd();
    iView--;
  } while(--iv>0);
  /*** write volume, etc... ***/
  CavTrailer();
  return 0;
}

/********************************************************
      SUBROUTINE PRJCTN
********************************************************/

void Prjctn(void) {
  int i;
  CavPlotPen(4);
  if (radi>0) {
    for(i=0;i<nPlt;i++){
      int ipl=iPlt[i];
      CavPlotArc(xyz[ipl][kx],xyz[ipl][ky],xyz[ipl][kz],radi,0,360,natm[ipl]);
    }
  }

  for(i=0;i<nBond;i++){
    int ips,ipe;
    ips=iPs[i];
    ipe=iPe[i];
    if (bExclFlag[ips]!=0 && bExclFlag[ipe]!=0) {
      /*    *** exclusive -- in Cavity *** */
      CavPlotPen(3);
    } else {
      /* **** Not Exclusive **** */
      CavPlotPen(2);
    }
    CavPlotLine(xyz[ips][kx],xyz[ips][ky],xyz[ips][kz],
	   xyz[ipe][kx],xyz[ipe][ky],xyz[ipe][kz],
	   natm[ips],natm[ipe]);
  }
}

static int CavMainLoop(void) {
  int iRetry;
  int iZNum,nSec;

  CavPlotPen(1);
  /*     *** Title *** */
  CavPlotTitle(title);
  if (iPlot!=0) Prjctn();
  CavPlotPen(1);
  CavPlotFlush();

  /*     *** Main Loop *** */
  CavPlotComment("Cavity Start");
  iRetry=0;

  CheckRange();
  vol=0;
  iZNum=0;
  /*     *** Main Loop *** */
  do {
    nSec=(zmax-zmin)/divz;
    zsec=zmin+divz*iZNum;
    sprintf(lbuf,"Z = %7.3f (%4d/%4d) IView=%1d",zsec,iZNum,nSec,iView);
    CavPlotComment(lbuf);
    CavPlotFlush();
    setpl();
    kPlot=nPlot;
    if (iCave!=0) kPlot=nCrcl;
    if (PlotVDW(nSec,iZNum)==0) continue;
    plvol=PlVol();
    if (iCave!=0 || nEx!=0) ICave();
    vol+=plvol;
    sprintf(lbuf,"Volume/Plane =%7.3f",plvol);
    CavPlotComment(lbuf);
    CavPlotFlush();
    iZNum++;
  } while (iZNum<=nSec);
  return 1;
}


/************************************/
int PlotVDW(int nSec,int iZNum) {
  int arcs,arce;
  int i,j,k,l;
  /*c      write(*,*)sr,sx,sy */

  if (sr<0) {
    return 0;
  }
  CavPlotPen(1);
  for(i=0;i<nPlot;i++){
    iUsePl[i]=0;
  }
  for(i=0;i<nPlot;i++){
    double arcx,arcy,arcr,xp,yp;
    int ll[MaxAtm];
    int lPlot;
    int lprev;
    arcs=arce=0;
    lPlot=0;
    for(k=0;k<kPlot;k++){
      if ((x[k]-x[i])*(x[k]-x[i])+(y[k]-y[i])*(y[k]-y[i])
	  <(r[k]+r[i])*(r[k]+r[i])) {
	ll[lPlot++]=k;
      }
    }
    arcx=x[i];
    arcy=y[i];
    arcr=r[i];
    lprev=0;
    for(j=0;j<=360;j++){
      xp=x[i]+r[i]*cost[j];
      yp=y[i]+r[i]*sint[j];
      if (xp<xmin || xp>xmax || yp<ymin || yp>ymax || j==360) goto L400;
      if (lprev!=i && 
	  (xp-x[lprev])*(xp-x[lprev])+(yp-y[lprev])*(yp-y[lprev])<r2[lprev]) {
	goto L400; 
      }

      /* for SPHERE Option */
      if (sr>0) {  
	if (sr<((xp-sx)*(xp-sx)+(yp-sy)*(yp-sy))) goto L400;
      }

      for(l=0;l<lPlot;l++){
	k=ll[l];
	  /*      for(k=0;k<kPlot;k++){*/
	if (k!=i) {
	  if ((xp-x[k])*(xp-x[k])+(yp-y[k])*(yp-y[k])<r2[k]) {
	    lprev=k;
	    goto L400; 
	  }
	}
      }
      if (nInt==0 && AutoExpandRange(nSec,iZNum,xp,yp)==0) return 0;
      continue;

    L400:
      arce=j;
      if (arce>arcs) {
	if (nInt==0 && AutoExpandRange(nSec,iZNum,xp,yp)==0) return 0;
	CavPlotColor(iCol[iNum[i]]);
	CavPlotArc(arcx,arcy,zsec,arcr,arcs,arce,natm[iNum[i]]);
	iUse[iNum[i]]=1;
	iUsePl[i]=1;
      }
      arcs=j+1;
    }
  }
  CavPlotColor(0);
  return 1;
}

double PlVol(void) {
  double divz3,vx,vy;
  double vflag;
  int i,j,k;
  int lastc;
  double plvol=0;
  if (iVol==0) return 0.0;
  j=0;
  for(i=0;i<nPlot;i++) {
    if (iUsePl[i]!=0) {
      iUsePn[j++]=i;
    }
  }
  
  divz3=divz*divz*divz;
  for(vx=xmin;vx<=xmax;vx+=divz) {
    lastc=0;
    for(vy=ymin;vy<=ymax;vy+=divz) {
      vflag=divz3;
      if (sr>0){
	if (((vx-sx)*(vx-sx)+(vy-sy)*(vy-sy))>sr) {
	  vflag=0;
	  goto L331;
	}
      }
      if ((vx-x[lastc])*(vx-x[lastc])+(vy-y[lastc])*(vy-y[lastc])
	  <=r2[lastc]) {
	vflag=0;
	goto L331;
      }
      
      for(i=0;i<j;i++){
	k=iUsePn[i];
	if ((vx-x[k])*(vx-x[k])+(vy-y[k])*(vy-y[k])<=r2[k]) {
	  lastc=k;
	  vflag=0;
	  goto L331;
	}
      }
      for(i=0;i<kPlot;i++){
	if ((vx-x[i])*(vx-x[i])+(vy-y[i])*(vy-y[i])<r2[i]) {
	  if (iUse[iNum[i]]==0) iUse[iNum[i]]=2;
	  lastc=i;
	  vflag=0;
	  goto L331;
	}
      }
    L331:
      plvol+=vflag;
    }
  }
  return plvol;
}
 
static int ICave(void) {
  int i,j,k;
  int arcs,arce;
  arcs=arce=0;
  CavPlotPen(3);
  for(i=nPlot;i<nCrcl;i++){
    double arcx,arcy,arcr,xp,yp;
    arcs=0;
    arcx=x[i];
    arcy=y[i];
    arcr=r[i];
    for(j=0;j<=360;j++){
      xp=x[i]+r[i]*cost[j];
      yp=y[i]+r[i]*sint[j];
      if (j==360) goto L800;
      if (xp<xmin || xp>xmax || yp<ymin || yp>ymax) goto L800;
      for(k=0;k<nCrcl;k++){
	if (k!=i) {
	  if ((xp-x[k])*(xp-x[k])+(yp-y[k])*(yp-y[k])<r2[k]) goto L800;
	}
      }
      break;
    L800:
      arce=j;
      if (arce>arcs) {
	CavPlotArc(arcx,arcy,arcr,zsec,arcs,arce,natm[iNum[i]]);
      }
      arcs=j+1;
    }
  }
  return 1;
}

/**************************************
      subroutine setpl
**************************************/
static void setpl(void) {
  int i,j,k;
  nPlot=0;
  sx=sxyz[kx];
  sy=sxyz[ky];
  sr=0;
  if (sradi!=0) sr=sradi*sradi-((sxyz[kz]-zsec)*(sxyz[kz]-zsec)); 
  for(i=0;i<nAtom;i++){
    double dist,dist2;
    if (xyz[i][kx]+w[i]<xmin || xyz[i][kx]-w[i]>xmax) continue;
    if (xyz[i][ky]+w[i]<ymin || xyz[i][ky]-w[i]>ymax) continue;
    if (bExclFlag[i]!=0) continue;
    dist=fabs(xyz[i][kz]-zsec);
    if (dist>w[i]) continue;
    dist2=dist*dist;
    x[nPlot]=xyz[i][kx];
    y[nPlot]=xyz[i][ky];
    name[nPlot]=natm[i];
    iNum[nPlot]=i;
    r2[nPlot]=w2[i]-dist2;
    r[nPlot]=sqrt(r2[nPlot]);
    nPlot++;
  }

  nEx=0;
  if (iCave!=0) {
    double dist,dist2;
    for(i=0;i<nExcl;i++){
      j=iExcl[i];
      dist=fabs(xyz[j][kz]-zsec);
      if (dist<=w[j]) {
	dist2=dist*dist;
	k=nPlot+nEx;
	nEx++;
	x[k]=xyz[j][kx];
	y[k]=xyz[j][ky];
	name[k]=natm[j];
	r2[k]=w2[j]-dist2;
	r[k]=sqrt(r2[k]);
      }
    }
  }
  nCrcl=nPlot+nEx;

  fprintf(l_log,"ZSEC=%8.2f,NATOM=%5d\n",zsec,nCrcl);
  if (iPrint!=0) {
    for(i=0;i<nCrcl;i++){
      fprintf(l_log,"%-6s%10.6f%10.6f%10.6f\n",name[i],x[i],y[i],r[i]);
    }
  }
}

/******************************************
      Function IAutEx(NSEC,L,XP,YP)
******************************************/
static int AutoExpandRange(int nSec,int l,double xp,double yp) {
  if (l==0) {
    zmin-=fIncSt*divz;
    fprintf(l_log,"RECALCULATION ZMIN =%6.2f\n",zmin);
    return 0;
  }
  if (cmin[kx]>xp) cmin[kx]=xp;
  if (cmax[kx]<xp) cmax[kx]=xp;
  if (cmin[ky]>yp) cmin[ky]=yp;
  if (cmax[ky]<yp) cmax[ky]=yp;
  if (cmin[kz]>zsec) cmin[kz]=zsec;
  if (cmax[kz]<zsec) cmax[kz]=zsec;
  if (l>=nSec) {
    zmax=zmax+fIncSt*divz;
    fprintf(l_log,"RECALCULATION ZMAX =%6.2f\n",zmax);
    return 0;
  }
  if (xp<xmin){
    xmin-=fIncSt*divz;
    fprintf(l_log,"RECALCULATION XMIN =%6.2f\n",xmin);
    return 0;
  } else if (xp>xmax) {
    xmax+=fIncSt*divz;
    fprintf(l_log,"RECALCULATION XMAX =%6.2f\n",xmax);
    return 0;
  } else if (yp<ymin) {
    fprintf(l_log,"RECALCULATION YMIN =%6.2f\n",ymin);
    ymin-=fIncSt*divz;
    return 0;
  } else if (yp>ymax) {
    fprintf(l_log,"RECALCULATION YMAX =%6.2f\n",ymax);
    ymax+=fIncSt*divz;
    return 0;
  }
  return 1;
}

/******************************************
      subroutine CheckRange
c
c     Automatic Range Expand Check
c
******************************************/
static void CheckRange(void) {
  if (nInt!=0) return;
  if (zmin<-fMaxAr) {
    error("Auto Range Over Flow / ZMIN !!");
  } else if (zmax>fMaxAr) {
    error("Auto Range Over Flow / ZMAX !!");
  } else if (xmin<-fMaxAr) {
    error("Auto Range Over Flow / XMIN !!");
  } else if (xmax>fMaxAr) {
    error("Auto Range Over Flow / XMAX !!");
  } else if (ymin<-fMaxAr) {
    error("Auto Range Over Flow / YMIN !!");
  } else if (ymax>fMaxAr) {
    error("Auto Range Over Flow / YMAX !!");
  }
}
