#include "cavity.h"
#include <stdlib.h>

/*
c     ------------------------
c         Common Variables
c     ------------------------
*/

/*    Title (in 80 characters) */
char title[80];

/*
c     Control Flags Group 1
c     iview  : View Angle
c     nint   : 
c     ivol   : Volume Calculation ? (do:1, don't:0)
c     iplot  : Plot Bonding ? (do:1, don't:0)
c     iprint :
c     icave  :
c     nplt   :
c     nbond  :
c     nexcl  :
*/
int iView,nInt,iVol,iPlot,iPrint,iCave,nPlt,nBond,nExcl;

/*
c     Control Flags Group 2
c     scale  : Plot Scale  ( usually 1 )
c     radi   : Atom Radius ( usually 0 )
c     ipen(1),ipen(2),...  :  Pen Number ( usually 0,1,2,3 ... )
*/
double scale,radi;
int *iPen;

/*
c     Control Flags Group 3 (Atoms, Bonds, Atoms in Cavity)
c     iplt(1), iplt(2), ... : Atom Numbers shown in the Figure
c     [ips(1),ipe(1)],[ips(2),ipe(2)], ... : Atom Numbers of both sides
c                                            of Bonds shown in the figure
c     iexcl(1), iexcl(2), ... : Atom Numbers in Cavity
*/

int *iPlt,*iPs,*iPe,*iExcl;

/*
c     Control Flags Group 4 (Van der Waals Radius + 1.2? Angstrome)
c     vdw(1),vdw(2), ... : Radius of Elements
*/
double *Vdw;

/*
c     Control Flags Group 5 (Calculation Range, Step in angstrome)
c     xmin, xmax, ymin, ymax, zmin, zmax : Calulation Range (used when nint=0)
c     divz   : Cavity/Volume Calculation Step
*/
double xmin,xmax,ymin,ymax,zmin,zmax,divz;

/*
c     Temporary Variables  (Atoms on one of the section of the Cavity)
c     x(), y() : Location
c     r : radius of atom circle
c     r2 : R*R
c     name : Atom Name
c     inum : Atom Number
c     nplot :
c     ncrcl :
c     natom :
c     nex   :
c     zsec  : Z location of this plain
*/

double *x,*y,*r,*r2;
char **name;
int *iNum,nPlot,kPlot,nCrcl,nAtom,nEx;
double sx,sy,sr;

/*
c     Atom Data(Location, Color, Radius, ... )
c     natm  : 
c     itbl  :
c     xyz   : Location of Atoms
c     w     :
c     w2    :
c     icol  : Color Number of Atoms
*/
char (*natm)[10];
int *iTbl,*iCol;
double (*xyz)[3],*w,*w2;

/*
c     Other Variables
c     sint, cost : sint(1)=sin(0 deg), ..., sint(n)=sin(n-1 deg)
c     kx, ky, kz : plot axis (from iview)
c     cmin, cmax : Plot Range
c     vmt : View Matrix
c     rvmt : View Matrix Inverse
c     org : Origin of Coordinate
c     lbuf : Line Buffer for Data Input
c     ierr : Error Flag
*/

double sint[361],cost[361],cmin[3],cmax[3],vmt[3][3],rvmt[3][3],org[3];
int kx,ky,kz;

char lbuf[1024];
int iErr;
double sxyz[3],sradi;

/*
c     Atoms Used to plot Cavity
c     iuse : Used Flag of Atom 
c     iusepl : Used Flag of Atom in the Plain
c     iusepn : 
*/
int *iUse,*bExclFlag,*iUsePl,*iUsePn;

/*
c     Cavity Volume, Cavity Volume in One of The Section, ...
c     vol : total volume
c     plvol : volume in one section
*/
double vol,plvol;

FILE *l_log;

int GlobalInit(void){
  iPen=malloc(sizeof(int)*MaxPen);
  if (iPen==NULL) return 0;
  iPlt=malloc(sizeof(int)*MaxPlt);
  if (iPlt==NULL) return 0;
  iPs=malloc(sizeof(int)*MaxBnd);
  if (iPs==NULL) return 0;
  iPe=malloc(sizeof(int)*MaxBnd);
  if (iPe==NULL) return 0;
  iExcl=malloc(sizeof(int)*MaxExc);
  if (iExcl==NULL) return 0;
  Vdw=malloc(sizeof(double)*MaxVDW);
  if (Vdw==NULL) return 0;
  x=malloc(sizeof(double)*MaxAPl);
  if (x==NULL) return 0;
  y=malloc(sizeof(double)*MaxAPl);
  if (y==NULL) return 0;
  r=malloc(sizeof(double)*MaxAPl);
  if (r==NULL) return 0;
  r2=malloc(sizeof(double)*MaxAPl);
  if (r2==NULL) return 0;
  name=malloc(sizeof(char *)*MaxAPl);
  if (name==NULL) return 0;
  iNum=malloc(sizeof(int)*MaxAPl);
  if (iNum==NULL) return 0;
  natm=malloc(sizeof(char)*10*MaxAtm);
  if (natm==NULL) return 0;
  iTbl=malloc(sizeof(int)*MaxAtm);
  if (iTbl==NULL) return 0;
  iCol=malloc(sizeof(int)*MaxAtm);
  if (iCol==NULL) return 0;
  xyz=malloc(sizeof(double)*3*MaxAtm);
  if (xyz==NULL) return 0;
  w=malloc(sizeof(double)*MaxAtm);
  if (w==NULL) return 0;
  w2=malloc(sizeof(double)*MaxAtm);
  if (w2==NULL) return 0;
  iUse=malloc(sizeof(int)*MaxAtm);
  if (iUse==NULL) return 0;
  bExclFlag=malloc(sizeof(int)*MaxAtm);
  if (bExclFlag==NULL) return 0;
  iUsePl=malloc(sizeof(int)*MaxAtm);
  if (iUsePl==NULL) return 0;
  iUsePn=malloc(sizeof(int)*MaxAPl);
  if (iUsePn==NULL) return 0;
  return 1;
}
